package net.jansensan.loaders
{
	// + ----------------------------------------
	//		[ IMPORTS ]
	// + ----------------------------------------
	
	import flash.display.Loader;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.net.URLRequest;
	import flash.system.ApplicationDomain;
	import flash.system.LoaderContext;
	
	
	/**
	 * <p>The AuthoringAssetsLoader class uses flash.display.Loader to specifically handle the loading of SWF files whose library contain items with export classes.</p>
	 * <p>The assets will be available to use in the whole application once the file is loaded, no need to retrieve any data from the AuthoringAssetsLoader.</p>
	 * 
	 * <b>Usage:</b>
	 * <p>
	 * function loadAssets():void
	 * {
	 * var assetsLoader:AuthoringAssetsLoader = new AuthoringAssetsLoader();<br/>
	 * assetsLoader.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);<br/>
	 * assetsLoader.addEventListener(AuthoringAssetsLoader.PROGRESS, progressHandler);<br/>
	 * assetsLoader.addEventListener(AuthoringAssetsLoader.READY, readyHandler);<br/>
	 * assetsLoader.load("someSWFFile.swf");<br/>
	 * }
	 * 
	 * @author Mat Janson Blanchet
	 */
	public class AuthoringAssetsLoader extends EventDispatcher
	{
		// + ----------------------------------------
		//		[ CONSTANTS ]
		// + ----------------------------------------
		
		// events
		public	static	const	PROGRESS		:String = "PROGRESS";
		public	static	const	READY			:String = "READY";
		public	static	const	LOADING_ABORTED	:String = "LOADING_ABORTED";
		
		
		// + ----------------------------------------
		//		[ VARIABLES ]
		// + ----------------------------------------
		
		// private / protected
		private	var	_assetsSWFURL	:String;
		private	var	_loader			:Loader;
		private	var	_isLoading		:Boolean = false;
		private	var	_loadedRatio	:Number = 0;
		
		
		// + ----------------------------------------
		//		[CONSTRUCTOR ]
		// + ----------------------------------------
		
		public function AuthoringAssetsLoader()
		{
			
		}
		
		
		// + ----------------------------------------
		//		[ PUBLIC METHODS ]
		// + ----------------------------------------
		
		/**
		 * Initiates the loading of an SWF file that ideally contains only a library of items with export classes.
		 * @param assetsSWFURL The path to the file, in string format. The AuthoringAssetsLoader will handle the conversion to URLRequest.
		 */
		public function load(assetsSWFURL:String):void
		{
			_assetsSWFURL = assetsSWFURL;
			_loader = new Loader();
			addLoaderListeners();
			var URL:URLRequest = new URLRequest(_assetsSWFURL);
			var loaderContext:LoaderContext = new LoaderContext(false, ApplicationDomain.currentDomain);
			_loader.load(URL, loaderContext);
			_isLoading = true;
		}
		
		
		/**
		 * Cancels the loading of the file.
		 */
		public function abortLoading():void
		{
			removeLoaderListeners();
			try
			{
				_loader.close();
			}
			catch(error:Error)
			{
				trace("\n", this, "--- abortLoading ---");
				trace(error.getStackTrace());
			}
			_isLoading = false;
		}
		
		
		// + ----------------------------------------
		//		[ PRIVATE/PROTECTED METHODS ]
		// + ----------------------------------------
		
		private function addLoaderListeners():void
		{
			_loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, loaderIOErrorHandler, false, 0, true);
			_loader.contentLoaderInfo.addEventListener(Event.COMPLETE, loaderCompleteHandler, false, 0, true);
		}
		
		
		private function removeLoaderListeners():void
		{
			_loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, loaderIOErrorHandler);
			_loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, loaderCompleteHandler);
		}
		
		
		// + ----------------------------------------
		//		[ EVENT HANDLERS ]
		// + ----------------------------------------
		
		private function loaderIOErrorHandler(event:IOErrorEvent):void
		{
			trace(this, "--- loaderIOErrorHandler ---");
			
			_isLoading = false;
			removeLoaderListeners();
			dispatchEvent(event);
		}
		
		
		private function loaderProgressHandler(event:ProgressEvent):void
		{
			if(event.bytesTotal > 0) _loadedRatio = event.bytesLoaded / event.bytesTotal;
			dispatchEvent(new Event(AuthoringAssetsLoader.PROGRESS));
		}
		
		
		private function loaderCompleteHandler(event:Event):void
		{
			_loadedRatio = 1;
			dispatchEvent(new Event(AuthoringAssetsLoader.PROGRESS));
			
			_isLoading = false;
			removeLoaderListeners();
			dispatchEvent(new Event(AuthoringAssetsLoader.READY));
		}
		
		
		// + ----------------------------------------
		//		[ GETTERS / SETTERS ]
		// + ----------------------------------------
		
		public function get loadedRatio():Number
		{
			return _loadedRatio;
		}
		
		
		public function get isLoading():Boolean
		{
			return _isLoading;
		}
		
		
	}
}