package net.jansensan.loaders
{
	// + ----------------------------------------
	//		[ IMPORTS ]
	// + ----------------------------------------
	
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.text.StyleSheet;
	
	
	/**
	 * <p>The CSSLoader class uses flash.net.URLLoader to specifically handle the loading of CSS files.</p>
	 * 
	 * <b>Usage:</b>
	 * <p>
	 * function loadCSS():void
	 * {
	 * var cssLoader:CSSLoader = new CSSLoader();<br/>
	 * cssLoader.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);<br/>
	 * cssLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityHandler);<br/>
	 * cssLoader.addEventListener(CSSLoader.PROGRESS, progressHandler);<br/>
	 * cssLoader.addEventListener(CSSLoader.READY, readyHandler);<br/>
	 * cssLoader.loadCSS("someCSSFile.css");<br/>
	 * }
	 * </p>
	 * function readyHandler():void
	 * {
	 * trace(cssLoader.styleSheet);<br/>
	 * }
	 * </p>
	 * 
	 * @see flash.net.URLLoader
	 * @author Mat Janson Blanchet
	 */
	public class CSSLoader extends EventDispatcher
	{
		// + ----------------------------------------
		//		[ CONSTANTS ]
		// + ----------------------------------------
		
		// events
		public	static	const	PROGRESS	:String = "PROGRESS";
		public	static	const	READY		:String = "READY";
		
		
		// + ----------------------------------------
		//		[ VARIABLES ]
		// + ----------------------------------------
		
		// private / protected
		private	var	_CSSURL			:String;
		private	var	_URLLoader		:URLLoader;
		private	var	_isLoading		:Boolean = false;
		private	var	_loadedRatio	:Number = 0;
		private	var	_styleSheet		:StyleSheet;
		
		
		// + ----------------------------------------
		//		[CONSTRUCTOR ]
		// + ----------------------------------------
		
		public function CSSLoader()
		{
			
		}
		
		
		// + ----------------------------------------
		//		[ PUBLIC METHODS ]
		// + ----------------------------------------
		
		/**
		 * Initiates the loading of a CSS file.
		 * @param CSSURL The path to the file, in string format. The CSSLoader will handle the conversion to URLRequest.
		 */
		public function load(CSSURL:String):void
		{
			_CSSURL = CSSURL;
			_URLLoader = new URLLoader();
			addURLLoaderListeners();
			_URLLoader.load(new URLRequest(_CSSURL));
			_isLoading = true;
		}
		
		
		/**
		 * Cancels the loading of the file.
		 */
		public function abortLoading():void
		{
			removeURLLoaderListeners();
			try
			{
				_URLLoader.close();
			}
			catch(error:Error)
			{
				trace("\n", this, "--- abortLoading ---");
				trace(error.getStackTrace());
			}
			_isLoading = false;
		}
		
		
		// + ----------------------------------------
		//		[ PRIVATE/PROTECTED METHODS ]
		// + ----------------------------------------
		
		private function addURLLoaderListeners():void
		{
			_URLLoader.addEventListener(IOErrorEvent.IO_ERROR, URLLoaderIOErrorHandler, false, 0, true);
			_URLLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, URLLoaderSecurityErrorHandler, false, 0, true);
			_URLLoader.addEventListener(ProgressEvent.PROGRESS, URLLoaderProgressHandler, false, 0, true);
			_URLLoader.addEventListener(Event.COMPLETE, URLLoaderCompleteHandler, false, 0, true);
		}
		
		
		private function removeURLLoaderListeners():void
		{
			_URLLoader.removeEventListener(IOErrorEvent.IO_ERROR, URLLoaderIOErrorHandler);
			_URLLoader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, URLLoaderSecurityErrorHandler);
			_URLLoader.removeEventListener(ProgressEvent.PROGRESS, URLLoaderProgressHandler);
			_URLLoader.removeEventListener(Event.COMPLETE, URLLoaderCompleteHandler);
		}
		
		
		private function getCSS():void
		{
			//trace(this, "--- getCSS ---");
			
			_styleSheet = new StyleSheet();
			_styleSheet.parseCSS(_URLLoader.data);
			/*
			trace("_styleSheet: " + _styleSheet);
			trace("_styleSheet.styleNames: " + _styleSheet.styleNames);
			*/
		}
		
		
		// + ----------------------------------------
		//		[ EVENT HANDLERS ]
		// + ----------------------------------------
		
		private function URLLoaderIOErrorHandler(event:IOErrorEvent):void
		{
			trace(this, "--- URLLoaderIOErrorHandler ---");
			trace(event);
			trace(event.text);
			
			_isLoading = false;
			removeURLLoaderListeners();
			dispatchEvent(event);
		}
		
		
		private function URLLoaderSecurityErrorHandler(event:SecurityErrorEvent):void
		{
			trace(this, "--- URLLoaderSecurityErrorHandler ---");
			trace(event);
			trace(event.text);
			
			_isLoading = false;
			removeURLLoaderListeners();
			dispatchEvent(event);
		}
		
		
		private function URLLoaderProgressHandler(event:ProgressEvent):void
		{
			if(event.bytesTotal > 0) _loadedRatio = event.bytesLoaded / event.bytesTotal;
			dispatchEvent(new Event(CSSLoader.PROGRESS));
		}
		
		
		private function URLLoaderCompleteHandler(event:Event):void
		{
			_loadedRatio = 1;
			dispatchEvent(new Event(CSSLoader.PROGRESS));
			
			_isLoading = false;
			removeURLLoaderListeners();
			getCSS();
			dispatchEvent(new Event(CSSLoader.READY));
		}
		
		
		// + ----------------------------------------
		//		[ GETTERS / SETTERS ]
		// + ----------------------------------------
		
		public function get styleSheet():StyleSheet
		{
			return _styleSheet;
		}
		
		
		public function get loadedRatio():Number
		{
			return _loadedRatio;
		}
		
		
		public function get isLoading():Boolean
		{
			return _isLoading;
		}
		
		
	}
}