package net.jansensan.loaders
{
	// + ----------------------------------------
	//		[ IMPORTS ]
	// + ----------------------------------------
	
	import flash.display.Loader;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.net.URLRequest;
	import flash.system.ApplicationDomain;
	import flash.system.LoaderContext;
	import flash.text.Font;
	
	
	/**
	 * <p>The SWFFontsLoader class uses flash.display.Loader to specifically handle the loading of SWF files that contain embedded fonts.</p>
	 * <p>Fonts will be available to use in the whole application once the file is loaded, no need to retrieve any data from the SWFFontsLoader.</p>
	 * 
	 * <b>Usage:</b>
	 * <p>
	 * function loadFonts():void
	 * {
	 * var swfFontsLoader:SWFFontsLoader = new SWFFontsLoader();<br/>
	 * swfFontsLoader.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);<br/>
	 * swfFontsLoader.addEventListener(SWFFontsLoader.PROGRESS, progressHandler);<br/>
	 * swfFontsLoader.addEventListener(SWFFontsLoader.READY, readyHandler);<br/>
	 * swfFontsLoader.load("someSWFFile.swf");<br/>
	 * }
	 * 
	 * @see flash.display.Loader
	 * @author Mat Janson Blanchet
	 */
	public class SWFFontsLoader extends EventDispatcher
	{
		// + ----------------------------------------
		//		[ CONSTANTS ]
		// + ----------------------------------------
		
		// events
		public	static	const	PROGRESS		:String = "PROGRESS";
		public	static	const	READY			:String = "READY";
		public	static	const	LOADING_ABORTED	:String = "LOADING_ABORTED";
		
		
		// + ----------------------------------------
		//		[ VARIABLES ]
		// + ----------------------------------------
		
		// private / protected
		private	var	_fontsSWFURL	:String;
		private	var	_loader			:Loader;
		private	var	_isLoading		:Boolean = false;
		private	var	_loadedRatio	:Number = 0;
		
		// static
		
		
		// + ----------------------------------------
		//		[CONSTRUCTOR ]
		// + ----------------------------------------
		
		public function SWFFontsLoader()
		{
			
		}
		
		
		// + ----------------------------------------
		//		[ PUBLIC METHODS ]
		// + ----------------------------------------
		
		/**
		 * Initiates the loading of an SWF file that should contain only embedded fonts.
		 * @param fontsSWFURL The path to the file, in string format. The SWFFontsLoader will handle the conversion to URLRequest.
		 */
		public function load(fontsSWFURL:String):void
		{
			_fontsSWFURL = fontsSWFURL;
			_loader = new Loader();
			addLoaderListeners();
			var URL:URLRequest = new URLRequest(_fontsSWFURL);
			var loaderContext:LoaderContext = new LoaderContext(false, ApplicationDomain.currentDomain);
			_loader.load(URL, loaderContext);
			_isLoading = true;
		}
		
		
		/**
		 * Cancels the loading of the file.
		 */
		public function abortLoading():void
		{
			removeLoaderListeners();
			try
			{
				_loader.close();
			}
			catch(error:Error)
			{
				trace("\n", this, "--- abortLoading ---");
				trace(error.getStackTrace());
			}
			_isLoading = false;
		}
		
		
		/**
		 * Traces all the fonts that are currently embedded and available in the current application domain.
		 */
		public function traceEmbeddedFonts():void
		{
			trace("\n", this, "--- traceEmbeddedFonts ---");
			
			var embeddedFonts:Array = Font.enumerateFonts(false);
			var font:Font
			var i:int = 0;
			var numLoops:int = embeddedFonts.length;
			for(i; i < numLoops; i++)
			{
				font = embeddedFonts[i];
				trace("\t[" + i + "]\nfontName:" + font.fontName + ",\nstyle: " + font.fontStyle + ",\ntype: " + font.fontType + "\n");
			}
		}
		
		
		// + ----------------------------------------
		//		[ PRIVATE/PROTECTED METHODS ]
		// + ----------------------------------------
		
		private function addLoaderListeners():void
		{
			_loader.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, loaderIOErrorHandler, false, 0, true);
			_loader.contentLoaderInfo.addEventListener(Event.COMPLETE, loaderCompleteHandler, false, 0, true);
		}
		
		
		private function removeLoaderListeners():void
		{
			_loader.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, loaderIOErrorHandler);
			_loader.contentLoaderInfo.removeEventListener(Event.COMPLETE, loaderCompleteHandler);
		}
		
		
		// + ----------------------------------------
		//		[ EVENT HANDLERS ]
		// + ----------------------------------------
		
		private function loaderIOErrorHandler(event:IOErrorEvent):void
		{
			trace(this, "--- loaderIOErrorHandler ---");
			trace(event);
			trace(event.text);
			
			_isLoading = true;
			removeLoaderListeners();
			dispatchEvent(event);
		}
		
		
		private function loaderProgressHandler(event:ProgressEvent):void
		{
			if(event.bytesTotal > 0) _loadedRatio = event.bytesLoaded / event.bytesTotal;
			dispatchEvent(new Event(SWFFontsLoader.PROGRESS));
		}
		
		
		private function loaderCompleteHandler(event:Event):void
		{
			_loadedRatio = 1;
			dispatchEvent(new Event(SWFFontsLoader.PROGRESS));
			
			_isLoading = false;
			removeLoaderListeners();
			dispatchEvent(new Event(SWFFontsLoader.READY));
		}
		
		
		// + ----------------------------------------
		//		[ GETTERS / SETTERS ]
		// + ----------------------------------------
		
		public function get loadedRatio():Number
		{
			return _loadedRatio;
		}
		
		
		public function get isLoading():Boolean
		{
			return _isLoading;
		}
		
		
	}
}